import datetime
from decimal import Decimal
import os

from ofxstatement.ui import UI

from ofxstatement_lloyds.plugin import LloydsPlugin


def test_sample() -> None:
    plugin = LloydsPlugin(UI(), {})
    here = os.path.dirname(__file__)
    sample_filename = os.path.join(here, "sample-statement.csv")

    parser = plugin.get_parser(sample_filename)

    statement = parser.parse()

    assert statement is not None
    assert len(statement.lines) == 6

    assert statement.lines[0].amount == Decimal('-8.99')
    assert statement.lines[3].amount == Decimal('2000')
    assert statement.lines[5].date == datetime.datetime(2023, 12, 1)
    assert statement.lines[2].memo == 'ADAGIO EUROS       202.40 VISAXR     1.16168 CD 1417 '

    assert statement.start_balance == Decimal("6039.58")
    assert statement.end_balance == Decimal("2040.59")
    assert statement.start_date == datetime.datetime(2023, 12, 1)
    assert statement.end_date == datetime.datetime(2024, 1, 15)

def sum2num(x, y):
    return x+y

def test_iop():
    h=2-5
    h=h+5*19+8
    assert h==100

    assert sum2num(5, 9) == 14



from datetime import date
from decimal import Decimal
from typing import Iterable, Optional, TextIO

from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import Statement, StatementLine, generate_unique_transaction_id

from ofxstatement.parser import CsvStatementParser


class LloydsPlugin(Plugin):
    """Lloyds plugin (for developers only)"""

    def get_parser(self, filename: str) -> "LloydsParser":
        f = open(filename, "r")
        return LloydsParser(f)



class LloydsParser(CsvStatementParser):
    mappings = {"date": 0, "memo": 4}
    date_format = "%d/%m/%Y"
    start_balance = None
    end_balance = None
    start_date = None
    end_date = None

    def __init__(self, fin: TextIO) -> None:
        super().__init__(fin)
        self.uids = set()

    def parse(self) -> Statement:
        stmt = super().parse()
        stmt.start_date = self.start_date
        stmt.start_balance = Decimal(self.start_balance)
        stmt.end_date = self.end_date
        stmt.end_balance = Decimal(self.end_balance)
        return stmt

    def parse_record(self, line: list[str]) -> Optional[StatementLine]:
        sline = super().parse_record(line)
        sline.id = generate_unique_transaction_id(sline, self.uids)
        debit = line[5]
        credit = line[6]
        balance = line[7]
        self.start_balance = balance
        self.start_date = sline.date
        if debit == '':
            debit = 0
        else:
            debit = self.parse_decimal(debit)
        if credit == '':
            credit = 0
        else:
            credit = self.parse_decimal(credit)
        sline.amount = (-debit + credit)
        
        if self.end_balance == None:
            self.end_balance = balance
        
        if self.end_date == None:
            self.end_date = sline.date


        return sline

    def split_records(self) -> Iterable[list[str]]:
        reader = super().split_records()
        next(reader)  #Skip first line
        return reader
